@@ -46,6 +46,7 @@
 #include <gtk/gtkmessagedialog.h>
 
 #include "nautilus-cd-burner.h"
+#include "make-iso.h"
 
 #ifndef HAVE_MKDTEMP
 #include "mkdtemp.h"
@@ -467,7 +468,7 @@ cancel_mkisofs (gpointer data)
 }
 
 int
-make_iso (const char *filename, const char *label)
+make_iso (const char *filename, const char *label, gboolean warn_low_space)
 {
 	GnomeVFSURI *uri;
 	char *filelist = NULL;
@@ -580,17 +581,24 @@ make_iso (const char *filename, const char *label)
 		g_free (dirname);
 	} else if (iso_size / statfs_buf.f_bsize >= statfs_buf.f_bavail / 2048) {
 		g_free (dirname);
-		dialog = gtk_message_dialog_new (cd_progress_get_window (),
-						 GTK_DIALOG_DESTROY_WITH_PARENT,
-						 GTK_MESSAGE_ERROR,
-						 GTK_BUTTONS_CLOSE,
-						 _("Not enough space to store CD image (%ld Megabytes needed)"),
-						 iso_size / 512);
-		gtk_window_set_title (GTK_WINDOW (dialog),
-				      _("Not enough space"));
-		gtk_dialog_run (GTK_DIALOG (dialog));
-		gtk_widget_destroy (dialog);
-		mkisofs_output.result = RESULT_ERROR;
+		if (warn_low_space) {
+			dialog = gtk_message_dialog_new (cd_progress_get_window (),
+							 GTK_DIALOG_DESTROY_WITH_PARENT,
+							 GTK_MESSAGE_ERROR,
+							 GTK_BUTTONS_CLOSE,
+							 ngettext("Not enough space to store CD image (%ld Megabyte needed)",
+								  "Not enough space to store CD image (%ld Megabytes needed)",
+								  iso_size / 512),
+							 iso_size / 512);
+			gtk_window_set_title (GTK_WINDOW (dialog),
+					      _("Not enough space"));
+			gtk_dialog_run (GTK_DIALOG (dialog));
+			gtk_widget_destroy (dialog);
+			mkisofs_output.result = RESULT_ERROR;
+		} else {
+			mkisofs_output.result = RESULT_RETRY;
+		}
+
 		goto cleanup;
 	}
 
