@@ -22,10 +22,12 @@
    Authors: Alexander Larsson <alexl@redhat.com>
 */
 #include <gtk/gtk.h>
+#include <gtk/gtkfilesel.h>
 #include <gtk/gtkseparatormenuitem.h>
 #include <glade/glade.h>
 #include <libgnomevfs/gnome-vfs.h>
 #include <eel/eel-vfs-extensions.h>
+#include <gconf/gconf-client.h>
 #include <time.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -303,6 +305,92 @@ insert_rwcd_request_cb (CDRecorder *cdrecorder, gboolean is_reload,
 	}
 }
 
+static gboolean
+media_type_matches (CDMediaType type, gboolean blank_cd)
+{
+	switch (type)
+	{
+	case CD_MEDIA_TYPE_ERROR:
+	case CD_MEDIA_TYPE_UNKNOWN:
+	case CD_MEDIA_TYPE_CD:
+	case CD_MEDIA_TYPE_DVD:
+	case CD_MEDIA_TYPE_DVD_RAM:
+		return FALSE;
+	case CD_MEDIA_TYPE_CDR:
+		return (blank_cd ? FALSE : TRUE);
+	case CD_MEDIA_TYPE_CDRW:
+		return TRUE;
+	case CD_MEDIA_TYPE_DVDR:
+		return (blank_cd ? FALSE : TRUE);
+	case CD_MEDIA_TYPE_DVDRW:
+		return TRUE;
+	case CD_MEDIA_TYPE_DVD_PLUS_R:
+		return (blank_cd ? FALSE : TRUE);
+	case CD_MEDIA_TYPE_DVD_PLUS_RW:
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
+static gboolean
+wait_for_insertion (CDDrive *rec, gboolean blank_cd)
+{
+	CDMediaType type;
+	gboolean reload;
+
+	reload = FALSE;
+	type = guess_media_type (rec->device);
+	if (type == CD_MEDIA_TYPE_UNKNOWN || type == CD_MEDIA_TYPE_ERROR) {
+		reload = TRUE;
+	}
+
+	while (!media_type_matches (type, blank_cd)) {
+		GtkWidget *dialog, *parent;
+		const char *msg, *title;
+		int res;
+
+		if (reload && blank_cd) {
+			msg = N_("Please insert a rewritable media in the drive tray.");
+			title = N_("Insert rewritable media");
+		} else if (reload && !blank_cd) {
+			msg = N_("Please insert a blank media in the drive tray.");
+			title = N_("Insert blank media");
+		} else if (blank_cd) {
+			msg = N_("Please replace the in-drive media by a rewritable media.");
+			title = N_("Reload rewritable media");
+		} else {
+			msg = N_("Please replace the in-drive media by a blank media.");
+			title = N_("Reload blank media");
+		}
+
+		parent = glade_xml_get_widget (xml, "cdr_dialog");
+		dialog = gtk_message_dialog_new (GTK_WINDOW (parent),
+				GTK_DIALOG_MODAL,
+				GTK_MESSAGE_INFO,
+				GTK_BUTTONS_OK_CANCEL,
+				"%s", _(msg));
+		gtk_dialog_set_default_response (GTK_DIALOG (dialog),
+				GTK_RESPONSE_OK);
+		gtk_window_set_title (GTK_WINDOW (dialog), _(title));
+		res = gtk_dialog_run (GTK_DIALOG (dialog));
+		gtk_widget_destroy (dialog);
+		if (res == GTK_RESPONSE_CANCEL) {
+			return FALSE;
+		}
+
+		type = guess_media_type (rec->device);
+		reload = FALSE;
+		if (type == CD_MEDIA_TYPE_UNKNOWN
+				|| type == CD_MEDIA_TYPE_ERROR) {
+			reload = TRUE;
+		}
+	}
+
+	return TRUE;
+}
+
+
 static void
 details_clicked (GtkButton *button, gpointer data)
 {
@@ -391,6 +479,48 @@ estimate_size_callback (const gchar *rel_path,
 	return TRUE;
 }
 
+gboolean
+is_audio (char *mime_type) 
+{
+	if (strcmp (mime_type, "audio/x-wav") == 0 || strcmp (mime_type, "audio/x-ulaw") == 0) {
+		return TRUE;
+	}
+	return FALSE;
+}
+
+static gboolean
+are_all_files_music (char *uri)
+{
+	GnomeVFSFileInfo *info;
+	GnomeVFSDirectoryHandle *handle;
+	GnomeVFSResult result;
+	gboolean all_music;
+
+	result = gnome_vfs_directory_open (&handle, uri, GNOME_VFS_FILE_INFO_FOLLOW_LINKS | GNOME_VFS_FILE_INFO_GET_MIME_TYPE);
+	if (result != GNOME_VFS_OK) {
+		return FALSE;
+	}
+	
+	info = gnome_vfs_file_info_new ();
+	all_music = TRUE;
+
+	while (TRUE) {
+		result = gnome_vfs_directory_read_next (handle, info);
+		if (result != GNOME_VFS_OK) {
+			break;
+		}
+		
+		if (!is_audio (info->mime_type)) {
+			all_music = FALSE;
+			break;
+		}
+	}
+
+	gnome_vfs_directory_close (handle);
+	gnome_vfs_file_info_unref (info);
+
+	return all_music;
+}
 
 static GnomeVFSFileSize
 estimate_size (char *uri)
@@ -401,7 +531,7 @@ estimate_size (char *uri)
 	gnome_vfs_directory_visit (uri,
 				   GNOME_VFS_FILE_INFO_DEFAULT,
 				   GNOME_VFS_DIRECTORY_VISIT_DEFAULT,
-				   estimate_size_callback,
+				   (GnomeVFSDirectoryVisitFunc) estimate_size_callback,
 				   &size);
 
 	return size;
@@ -421,7 +551,7 @@ setup_close_button (void)
 }
 
 static int
-burn_cd (CDDrive *rec, const char *source_iso, const char *label, gboolean eject_cd, gboolean blank_cd, int speed, gboolean dummy)
+burn_cd (CDDrive *rec, const char *source_iso, const char *label, gboolean eject_cd, gboolean blank_cd, gboolean music_cd, int speed, gboolean dummy)
 {
 	CDRecorder *cdrecorder;
 	GtkWidget *filesel;
@@ -430,13 +560,15 @@ burn_cd (CDDrive *rec, const char *source_iso, const char *label, gboolean eject
 	int fd;
 	int res = RESULT_ERROR;
 
+	filename = NULL;
+
 	if (rec->type == CDDRIVE_TYPE_FILE) {
 		if (source_iso != NULL) {
 			/* TODO: display some message here */
 			g_print ("Writing ISO onto ISO, you freak!\n");
 			return RESULT_FINISHED;
 		}
-		filesel = gtk_file_selection_new (_("Choose a filename for the cdrom image"));
+		filesel = gtk_file_selection_new (_("Choose a filename for the CD-ROM image"));
 		gtk_file_selection_set_filename (GTK_FILE_SELECTION (filesel),
 						 "~/image.iso");
 
@@ -449,21 +581,50 @@ burn_cd (CDDrive *rec, const char *source_iso, const char *label, gboolean eject
 			return RESULT_CANCEL;
 		}
 	} else {
-		if (source_iso == NULL) {
-			filename = g_build_filename (g_get_tmp_dir (), "image.iso.XXXXXX", NULL);
-			fd = mkstemp(filename);
-			close (fd);
-		} else {
+		if (source_iso == NULL && !music_cd) {
+			GConfClient *gc;
+			const char *path;
+
+			gc = gconf_client_get_default ();
+			path = gconf_client_get_string (gc, "/apps/nautilus-cd-burner/temp_iso_dir", NULL);
+			g_object_unref (G_OBJECT (gc));
+			if (path == NULL || strcmp (path, "") == 0) {
+				filename = g_build_filename (path, "image.iso.XXXXXX", NULL);
+				fd = mkstemp(filename);
+				close (fd);
+			}
+		} else if (!music_cd) {
 			filename = g_strdup (source_iso);
 		}
 	}
 
-	if (source_iso == NULL) {
-		/* TODO: Check for availible diskspace */
-		res = make_iso (filename, label);
-		if (!res) {
-			/* User cancelled or we had an error. */
-			goto out;
+	if (source_iso == NULL && !music_cd) {
+		if (filename != NULL) {
+			res = make_iso (filename, label, FALSE);
+		} else {
+			res = RESULT_RETRY;
+		}
+
+		if (res == RESULT_RETRY) {
+			g_free (filename);
+			filename = g_build_filename (g_get_tmp_dir (), "image.iso.XXXXXX", NULL);
+			fd = mkstemp(filename);
+			close (fd);
+
+			res = make_iso (filename, label, FALSE);
+			if (res == RESULT_RETRY) {
+				g_free (filename);
+				filename = g_build_filename (g_get_home_dir (), ".image.iso.XXXXXX", NULL);
+				fd = mkstemp(filename);
+				close (fd);
+
+				res = make_iso (filename, label, TRUE);
+			}
+
+			if (!res) {
+				/* User cancelled or we had an error. */
+				goto out;
+			}
 		}
 	}
 
@@ -474,8 +635,12 @@ burn_cd (CDDrive *rec, const char *source_iso, const char *label, gboolean eject
 		setup_close_button ();
 		gtk_main ();
 	} else {
+		if (!wait_for_insertion (rec, blank_cd)) {
+			return RESULT_CANCEL;
+		}
+
 		cdrecorder = cd_recorder_new ();
-		
+
 		g_signal_connect (G_OBJECT (cdrecorder),
 				  "progress-changed",
 				  G_CALLBACK (progress_changed_cb),
@@ -500,13 +665,11 @@ burn_cd (CDDrive *rec, const char *source_iso, const char *label, gboolean eject
 				  "insert-rwcd-request",
 				  G_CALLBACK (insert_rwcd_request_cb),
 				  NULL);
-		res = cd_recorder_write_iso (cdrecorder, rec, filename, speed, eject_cd, blank_cd, dummy);
+		res = cd_recorder_write_iso (cdrecorder, rec, filename, speed, eject_cd, blank_cd, music_cd, dummy);
 		if (source_iso == NULL) {
 			unlink (filename);
 		}
 
-		g_print ("res: %d\n", res);
-		
 		if (res == RESULT_FINISHED) {
 			cd_progress_set_text (_("Complete"));
 			setup_close_button ();
@@ -530,7 +693,7 @@ lookup_current_recorder (GladeXML *xml)
 	GtkWidget *option_menu;
 	GtkWidget *item;
 	CDDrive *rec;
-	
+
 	option_menu = glade_xml_get_widget (xml, "target_optionmenu");
 	item = gtk_menu_get_active (GTK_MENU (GTK_OPTION_MENU (option_menu)->menu));
 	rec = g_object_get_data (G_OBJECT (item), "cd-drive");
@@ -547,9 +710,12 @@ refresh_dialog (GladeXML *xml)
 	GtkWidget *item;
 	GtkWidget *eject_check;
 	GtkWidget *blank_check;
+	GtkWidget *music_check;
+	GtkWidget *name_entry;
 	GList *l, *children;
+	GConfClient *gc;
 	char *name;
-	int i;
+	int i, default_speed;
 	CDDrive *rec;
 
 	/* Find active recorder: */
@@ -566,6 +732,10 @@ refresh_dialog (GladeXML *xml)
 	}
 	g_list_free (children);
 
+	gc = gconf_client_get_default ();
+	default_speed = gconf_client_get_int (gc, "/apps/nautilus-cd-burner/default_speed", NULL);
+	g_object_unref (G_OBJECT (gc));
+
 	item = gtk_menu_item_new_with_label (_("Maximum Possible"));
 	g_object_set_data (G_OBJECT (item), "speed", GINT_TO_POINTER (0));
 	gtk_widget_show (item);
@@ -579,7 +749,7 @@ refresh_dialog (GladeXML *xml)
 		g_free (name);
 	}
 	/* Default to automatic */
-	gtk_option_menu_set_history (GTK_OPTION_MENU (option_menu), 0);
+	gtk_option_menu_set_history (GTK_OPTION_MENU (option_menu), MIN (default_speed, default_speed));
 
 	/* Disable speed if max speed < 1 */
 	gtk_widget_set_sensitive (option_menu, rec->max_speed_write > 0);
@@ -590,8 +760,17 @@ refresh_dialog (GladeXML *xml)
 
 	/* Disable blank for burn-to-iso */
 	blank_check = glade_xml_get_widget (xml, "blank_check");
-	gtk_widget_set_sensitive (blank_check, rec->type != CDDRIVE_TYPE_FILE);
-	
+	gtk_widget_set_sensitive (blank_check, rec->type & CDDRIVE_TYPE_CDRW_RECORDER);
+
+	/* Disable music for burn-to-iso */
+	music_check = glade_xml_get_widget (xml, "music_check");
+	gtk_widget_set_sensitive (music_check, rec->type != CDDRIVE_TYPE_FILE);
+
+	name_entry = glade_xml_get_widget (xml, "cdname_entry");
+	gtk_widget_set_sensitive (name_entry,
+				  rec->type == CDDRIVE_TYPE_FILE ||
+				  !gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (music_check)));
+
 }
 
 static void
@@ -602,6 +781,15 @@ target_changed (GtkOptionMenu *option_menu, gpointer data)
   refresh_dialog (xml);
 }
 
+static void
+music_cd_toggled (GtkToggleButton *button, gpointer data)
+{
+  GladeXML *xml = data;
+
+  refresh_dialog (xml);
+}
+
+
 static void
 init_dialog (GladeXML *xml, GtkWidget *dialog, GnomeVFSFileSize size, char *disc_name, gboolean disable_reuse)
 {
@@ -613,6 +801,7 @@ init_dialog (GladeXML *xml, GtkWidget *dialog, GnomeVFSFileSize size, char *disc
 	GtkWidget *label;
 	GtkWidget *entry;
 	GtkWidget *reuse;
+	GtkWidget *music_check;
 	struct tm *tm;
 	int size_mb;
 	char *str;
@@ -655,6 +844,10 @@ init_dialog (GladeXML *xml, GtkWidget *dialog, GnomeVFSFileSize size, char *disc
 	} else {
 		time (&now);
 		tm = localtime (&now);
+		/*
+		translators: see strftime man page for meaning of %b, %d and %Y
+		the maximum length for this field is 32 characters
+		*/
 		strftime (buf, sizeof (buf), _("Personal Data, %b %d, %Y"), tm);	
 		gtk_entry_set_text (GTK_ENTRY (entry), buf);
 	}
@@ -664,6 +857,13 @@ init_dialog (GladeXML *xml, GtkWidget *dialog, GnomeVFSFileSize size, char *disc
 	gtk_widget_set_sensitive (reuse, !disable_reuse);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (reuse), TRUE);
 
+	music_check = glade_xml_get_widget (xml, "music_check");
+	if (!are_all_files_music ("burn:///")) {
+		gtk_widget_hide (music_check);	
+	}
+	g_signal_connect (music_check, "toggled", (GCallback)music_cd_toggled, xml);
+
+	
 	refresh_dialog (xml);
 }
 
@@ -772,19 +972,34 @@ verify_iso (const char *filename, char **iso_label)
 	return TRUE;
 }
 
+static GdkPixbuf *
+my_gdk_pixbuf_new_from_stock (const char *name)
+{
+	GtkWidget *label;
+	GdkPixbuf *icon;
+
+	label = gtk_label_new ("");
+	icon = gtk_widget_render_icon (label, name, GTK_ICON_SIZE_BUTTON, NULL);
+	gtk_widget_destroy (label);
+
+	return icon;
+}
+
 int
 main (int argc, char *argv[])
 {
 	CDDrive *rec;
-	GtkWidget *dialog, *entry, *eject_check, *blank_check, *reuse_check, *progress_window;
+	GtkWidget *dialog, *entry, *eject_check, *blank_check, *music_check, *reuse_check, *progress_window;
 	GtkWidget *cancel_button, *dummy_check, *option_menu, *item;
 	const char *label;
 	int res, speed;
-	gboolean eject, blank, reuse, dummy;
+	gboolean eject, blank, music, reuse, dummy;
 	GnomeVFSURI *uri;
 	GnomeVFSFileSize size;
 	char *source_iso, *disc_name = NULL;
 	struct stat stat_buf;
+	GConfClient *gc;
+	GdkPixbuf *icon;
 
 	bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
@@ -866,7 +1081,10 @@ main (int argc, char *argv[])
 #ifndef ENABLE_DUMMY_WRITE
 	gtk_widget_hide (dummy_check);
 #endif
-	
+
+	icon = my_gdk_pixbuf_new_from_stock ("gtk-cdrom");
+	gtk_window_set_icon (GTK_WINDOW (dialog), icon);
+	gdk_pixbuf_unref (icon);
 	res = gtk_dialog_run (GTK_DIALOG (dialog));
 
 	gtk_widget_hide (dialog);
@@ -889,6 +1107,10 @@ main (int argc, char *argv[])
 		blank_check = glade_xml_get_widget (xml, "blank_check");
 		blank = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (blank_check));
 
+		music_check = glade_xml_get_widget (xml, "music_check");
+		music = (rec->type != CDDRIVE_TYPE_FILE) &&
+			gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (music_check));
+	
 		dummy = FALSE;
 
 #ifdef ENABLE_DUMMY_WRITE
@@ -898,8 +1120,11 @@ main (int argc, char *argv[])
 		option_menu = glade_xml_get_widget (xml, "speed_optionmenu");
 		item = gtk_menu_get_active (GTK_MENU (gtk_option_menu_get_menu (GTK_OPTION_MENU (option_menu))));
 		speed = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (item), "speed"));
+		gc = gconf_client_get_default ();
+		gconf_client_set_int (gc, "/apps/nautilus-cd-burner/default_speed", speed, NULL);
+		g_object_unref (G_OBJECT (gc));
 
-		res = burn_cd (rec, source_iso, label, eject, blank, speed, dummy);
+		res = burn_cd (rec, source_iso, label, eject, blank, music, speed, dummy);
 
 		reuse_check = glade_xml_get_widget (xml, "reuse_check");
 		reuse = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (reuse_check));
